<?php

namespace Drupal\monarch_media_canonical_download\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Field\BaseFieldDefinition;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for COS Media as File routes.
 */
class MediaCanonicalDownloadController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->configFactory = $container->get('config.factory');

    return $instance;
  }

  /**
   * Builds the response.
   */
  public function build($media = NULL) {
    $config = $this->configFactory->get('monarch_media_canonical_download.settings');
    $settings = $config->get($media->bundle());
    $enabled = $settings['enabled'] ?? FALSE;
    $overrides = $settings['overrides'] ?? [];

    foreach ($media->getFieldDefinitions() as $field_name => $definition) {
      if (!($definition instanceof BaseFieldDefinition) && ($field = $media->get($field_name)) instanceof FileFieldItemList) {
        if (
          ($file = $field->entity ?? NULL) &&
          file_exists($uri = $file->uri->value ?? NULL)
        ) {

          $mime = $file->getMimeType();
          $ext = pathinfo($uri, PATHINFO_EXTENSION);

          if ($enabled xor (($ext && in_array($ext, $overrides)) || ($mime && in_array($mime, $overrides)))) {
            return new BinaryFileResponse($uri, 200, [
              'Content-Type' => $file->getMimeType(),
              'Content-Length' => $file->getSize(),
              'Cache-Control' => 'private',
            ]);
          }
        }
      }
    }

    return $this->entityTypeManager->getViewBuilder('media')->view($media);
  }

}
